## Per-host rate limiting for asynchronous web scraping
[![TravisCI](https://travis-ci.org/djkelleher/pyppeteer-cluster.svg?branch=master)](https://travis-ci.org/github/djkelleher/pyppeteer-cluster)
[![codecov](https://codecov.io/gh/djkelleher/pyppeteer-cluster/branch/master/graph/badge.svg?token=IWXL8WQT3N)](https://codecov.io/gh/djkelleher/pyppeteer-cluster)
[![Python 3.7](https://img.shields.io/badge/python-3.6-blue.svg)](https://www.python.org/downloads/release/python-360/)
[![Python 3.7](https://img.shields.io/badge/python-3.7-blue.svg)](https://www.python.org/downloads/release/python-370/)
[![Python 3.8](https://img.shields.io/badge/python-3.8-blue.svg)](https://www.python.org/downloads/release/python-380/)
[![Python 3.9](https://img.shields.io/badge/python-3.9-blue.svg)](https://www.python.org/downloads/release/python-390/)

### Usage
```python
from host_rate_limit import RateLimiter

host_requests_per_sec = {
    'www.amazon.com': (1.25, 3), # wait between 1/3 and 1/1.25 seconds between dispatching requests to amazon.com
    'www.ebay.com': 0.33, # wait 1/0.33 second between dispatching requests to amazon.com
    'twitter.com': 1, # send 1 request per second to twitter.com
}

rate_limiter = RateLimiter(default_rate_limit=(1,2), # send between 1 and 2 requests per second to every host that is not listed in host_requests_per_sec
                           host_requests_per_sec=host_requests_per_sec)

async def request(url):
    await rate_limiter.maybe_sleep(url)
    # request would be dispatched now..
    # response = await http_client.get(url)

# asynchronously dispatch a batch of requests.
# urls: List[str]
await asyncio.gather(
    *[asyncio.create_task(request(url)) for url in urls])
```